import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math' as math;

class CubeTransition extends CruiserTransition
{
    final Curve curve;
    final double perspectiveScale;
    final double angle;
    final AlignmentGeometry enterPageAlignment;
    final AlignmentGeometry exitPageAlignment;
    final Color backgroundColor;

    CubeTransition({
        this.curve = Curves.easeOutSine,
        this.perspectiveScale = 0.001,
        double angle = 90,
        this.enterPageAlignment = Alignment.centerLeft,
        this.exitPageAlignment = Alignment.centerRight,
        this.backgroundColor = Colors.black,
    }) : this.angle = math.pi / 180 * angle;

    @override
    Widget build(BuildContext context, Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child) 
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;

        double width = MediaQuery.of(context).size.width;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Transform(
                        alignment: exitPageAlignment,
                        transform: Matrix4.identity()
                            ..setEntry(3, 2, perspectiveScale)
                            ..rotateY(angle * (animation.value))
                            ..leftTranslate(-width * (animation.value)),
                        child: child,
                );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Transform(
                alignment: enterPageAlignment,
                transform: Matrix4.identity()
                    ..setEntry(3, 2, perspectiveScale)
                    ..rotateY(-angle * (1 - animation.value))
                    ..leftTranslate(width * (1 - animation.value)),
                    child: child,
            );
        }
    }
}