
import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math' as math;

class FlipHorizontalTransition extends CruiserTransition 
{
    final Curve curve;
    final Color backgroundColor;
    final AlignmentGeometry enterPageAlignment;
    final AlignmentGeometry exitPageAlignment;
    final bool reverse;
    final double perspectiveScale;

    FlipHorizontalTransition({
        this.curve = Curves.easeOutSine,
        this.backgroundColor = Colors.black,
        this.enterPageAlignment = Alignment.center,
        this.exitPageAlignment = Alignment.center,
        this.reverse = false,
        this.perspectiveScale = 0.002,
    });

    @override
    Widget build(BuildContext context, Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child) 
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            // exits
            return Container(
                color: backgroundColor,
                child: animation.value > 0.5
                ? Container()
                : Transform(
                    alignment: exitPageAlignment,
                    transform: Matrix4.identity()
                        ..setEntry(3, 2, perspectiveScale)
                        ..rotateY(math.pi * animation.value * (reverse ? -1 : 1)),
                    child: child,
                )
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return animation.value > 0.5
                ? Transform(
                    alignment: enterPageAlignment,
                    transform: Matrix4.identity()
                    ..setEntry(3, 2, perspectiveScale)
                    ..rotateY(
                    -math.pi * (1 - animation.value) * (reverse ? -1 : 1)),
                    child: child,
                )
                : Container();
        }
    }
}
