
import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'dart:math' as math;

class FlipVerticalTransition extends CruiserTransition
{
    final Curve curve;
    final Color backgroundColor;
    final AlignmentGeometry enterPageAlignment;
    final AlignmentGeometry exitPageAlignment;
    final bool reverse;

    FlipVerticalTransition({
        this.curve = Curves.easeOutSine,
        this.backgroundColor = Colors.black,
        this.enterPageAlignment = Alignment.center,
        this.exitPageAlignment = Alignment.center,
        this.reverse = false,
    });

    @override
    Widget build(BuildContext context, Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child) 
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Container(
                color: backgroundColor,
                child: Opacity(
                    opacity:  animation.value > 0.5 ? 0 : 1,
                    child: Transform(
                        alignment: exitPageAlignment,
                        transform: Matrix4.identity()..setEntry(3, 2, 0.002)..rotateX(math.pi * animation.value * (reverse ? -1 : 1)), 
                        child: child
                    )
                )
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Opacity(opacity: animation.value > 0.5 ? 1 : 0,
                child: Transform(alignment: enterPageAlignment, 
                    transform: Matrix4.identity()..setEntry(3, 2, 0.002)..rotateX(-math.pi * (1 - animation.value) * (reverse ? -1 : 1)),
                    child: child
                )
            );

        }
    }
}
