
import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ForegroundToBackgroundTransition extends CruiserTransition
{
    final Curve curve;
    final Color backgroundColor;
    final double scale;

    ForegroundToBackgroundTransition({
        this.curve = Curves.easeOutSine,
        this.backgroundColor = Colors.black,
        this.scale = 0.5,
    });

    @override
    Widget build(BuildContext context, Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child) 
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;
        final width = MediaQuery.of(context).size.width;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Container(
                color: backgroundColor,
                child: Transform(
                alignment: Alignment.center,
                transform: Matrix4.identity()
                    ..scale(1.0 - (1.0 - scale) * animation.value)
                    ..leftTranslate(-width * animation.value),
                child: child,
                )
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Transform(
                transform: Matrix4.identity()..translate(width * (1.0 - animation.value)),
                child: child,
            );
        }
    }
}
