import 'dart:math';

import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ParallaxLeft extends CruiserTransition
{
    final Curve curve;

    ParallaxLeft({
        this.curve = Curves.easeOutQuart,
    });

    @override
    Widget build(BuildContext context,  Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child)
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;

        double width = MediaQuery.of(context).size.width;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Stack(
                children: [
                    Transform(
                        transform: Matrix4.identity()..translate(-width / 4 * animation.value),
                        child: child,
                    ),
                    Opacity(
                        opacity: min(0.5, secondaryAnimation.value),
                        child: Container(
                            width: width,
                            height: MediaQuery.of(context).size.height,
                            color: Colors.black,
                        )
                    )
                ],
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Transform(
                transform: Matrix4.identity()..translate(width * (1.0 - animation.value)),
                child: child,
            );
        }
    }
}