import 'dart:math';

import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class ParallaxTop extends CruiserTransition
{
    final Curve curve;

    ParallaxTop({
        this.curve = Curves.easeOutQuart,
    });

    @override
    Widget build(BuildContext context,  Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child)
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;

        var size = MediaQuery.of(context).size;

        double width = size.width;
        double height = size.height;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Stack(
                children: [
                    Transform(
                        transform: Matrix4.identity()..translate(0.0, -height / 4 * animation.value, 1),
                        child: child,
                    ),
                    Opacity(
                        opacity: min(0.5, secondaryAnimation.value),
                        child: Container(
                            width: width,
                            height: height,
                            color: Colors.black,
                        )
                    )
                ],
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Transform(
                transform: Matrix4.identity()..translate(0.0, height * (1.0 - animation.value), 1),
                child: child,
            );
        }
    }
}