import 'package:cruiser/src/models/cruiser_transition.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';

class SlideTransition extends CruiserTransition
{
   final Curve curve;
    final AlignmentGeometry enterPageAlignment;
    final AlignmentGeometry exitPageAlignment;
    final Color backgroundColor;

    SlideTransition({
        this.curve = Curves.easeOutSine,
        this.enterPageAlignment = Alignment.center,
        this.exitPageAlignment = Alignment.center,
        this.backgroundColor = Colors.black,
    });

    @override
    Widget build(BuildContext context, Animation<double> primaryAnimation, Animation<double> secondaryAnimation, Widget child) 
    {
        var exitAnimation = secondaryAnimation as ProxyAnimation;
        final double width = MediaQuery.of(context).size.width;

        if(!exitAnimation.isDismissed)
        {
            var animation = CurvedAnimation(parent: secondaryAnimation, curve: curve);

            return Container(
                color: backgroundColor,
                child: Transform(
                    alignment: exitPageAlignment,
                    transform: Matrix4.identity()
                        ..leftTranslate(-width * animation.value),
                    child: child,
                )
            );
        }
        else
        {
            var animation = CurvedAnimation(parent: primaryAnimation, curve: curve);

            return Transform(
                alignment: enterPageAlignment,
                transform: Matrix4.identity()
                    ..leftTranslate(width * (1 - animation.value)),
                child: child,
            );
        }
    }
}